@@ -119,7 +119,7 @@ static double siril_gsl_vector_sum(const gsl_vector *v) {
 	return sum;
 }
 
-static gboolean computeBackground_RBF(GSList *list, double *background, int channel, unsigned int width, unsigned int height, double smoothing, gchar **err) {
+static gboolean computeBackground_RBF(GSList *list, double *background, int channel, unsigned int width, unsigned int height, double smoothing, gchar **err, gboolean processing) {
 	/* Implementation of RBF interpolation with a thin-plate Kernel k(r) = r^2 * log(r)
 	
 	References:
@@ -138,6 +138,11 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 		Deterministic Computer Models. AIAA journal, 43(4):853–863, 2005.
 	*/
 
+	if (processing) {
+		char *msg = siril_log_color_message(_("RBF Extraction: processing channel %d...\n"), "green", channel);
+		msg[strlen(msg) - 1] = '\0';
+		set_progress_bar_data(msg, PROGRESS_RESET);
+	}
 
 	double pixel;
 	gsl_matrix *K;
@@ -209,10 +214,32 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 
 	/* Solve K*coef = f for coef */
 
-	int s;
+	int s, status;
 	gsl_permutation *p = gsl_permutation_alloc(n + 1);
-	gsl_linalg_LU_decomp(K, p, &s);
-	gsl_linalg_LU_solve(K, p, f, coef);
+	status = gsl_linalg_LU_decomp(K, p, &s);
+	if (status) {
+		siril_log_color_message("Error in RBF algorithm: %s\n", "red", gsl_strerror(status));
+		gsl_permutation_free(p);
+		gsl_matrix_free(K);
+		gsl_vector_free(f);
+		gsl_vector_free(coef);
+		free(background_scaled);
+		free(kernel_scaled);
+		free(list_array);
+		return FALSE;
+	}
+	status = gsl_linalg_LU_solve(K, p, f, coef);
+	if (status) {
+		siril_log_color_message("Error in RBF algorithm: %s\n", "red", gsl_strerror(status));
+		gsl_permutation_free(p);
+		gsl_matrix_free(K);
+		gsl_vector_free(f);
+		gsl_vector_free(coef);
+		free(background_scaled);
+		free(kernel_scaled);
+		free(list_array);
+		return FALSE;
+	}
 
 	/* precompute the kernel in the x>0, y>0 plane */
 	for (int i = 0; i < height_scaled; i++) {
@@ -227,6 +254,8 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 	}
 
 	/* Calculate background from coefficients coef */
+	double total = height_scaled * width_scaled;
+	int progress = 0;
 
 #pragma omp parallel shared(background_scaled) private(A) num_threads(com.max_thread)
 	{
@@ -246,6 +275,17 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 				pixel = siril_gsl_vector_sum(A);
 				background_scaled[j + i * width_scaled] = pixel;
 
+				if (processing) {
+#ifdef _OPENMP
+#pragma omp critical
+#endif
+					{
+						++progress;
+						if (!(progress % 32)) {
+							set_progress_bar_data(NULL,	(double) progress / total);
+						}
+					}
+				}
 			}
 		}
 		gsl_vector_free(A);
@@ -261,6 +301,10 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 	free(kernel_scaled);
 	free(list_array);
 
+	if (processing) {
+		set_progress_bar_data(PROGRESS_TEXT_RESET, PROGRESS_RESET);
+	}
+
 	return TRUE;
 }
 
@@ -809,16 +853,36 @@ void generate_background_samples(int nb_of_samples, double tolerance) {
 	redraw(REDRAW_OVERLAY);
 }
 
+static gboolean end_background(gpointer p) {
+	struct background_data *args = (struct background_data *)p;
+	stop_processing_thread();
+	if (args) {
+		invalidate_stats_from_fit(args->fit);
+		background_computed = TRUE;
+		if (!args->from_ui) {
+			free_background_sample_list(com.grad_samples);
+			com.grad_samples = NULL;
+		}
+		gtk_widget_set_sensitive(lookup_widget("background_ok_button"), TRUE);
+		adjust_cutoff_from_updated_gfit();
+		redraw(REMAP_ALL);
+		set_cursor_waiting(FALSE);
+		free(args);
+	}
+	return FALSE;
+}
+
 
 /* uses samples from com.grad_samples */
-gboolean remove_gradient_from_image(int correction, poly_order degree, double smoothing, gboolean use_dither, int interpolation_method) {
+gpointer remove_gradient_from_image(gpointer p) {
+	struct background_data *args = (struct background_data *)p;
 	gchar *error;
 	double *background = (double*)malloc(gfit.ry * gfit.rx * sizeof(double));
 	
 	if (!background && !com.script) {
 		PRINT_ALLOC_ERR;
 		set_cursor_waiting(FALSE);
-		return FALSE;
+		return NULL;
 	}
 
 	const size_t n = gfit.naxes[0] * gfit.naxes[1];
@@ -826,7 +890,7 @@ gboolean remove_gradient_from_image(int correction, poly_order degree, double sm
 	if (!image) {
 		free(background);
 		PRINT_ALLOC_ERR;
-		return FALSE;
+		return NULL;
 	}
 	
 	/* Make sure to update local median. Useful if undo is pressed */
@@ -838,36 +902,37 @@ gboolean remove_gradient_from_image(int correction, poly_order degree, double sm
 	for (int channel = 0; channel < gfit.naxes[2]; channel++) {
 		/* compute background */
 		gboolean interpolation_worked = TRUE;
-		if (interpolation_method == INTER_POLY){
-			interpolation_worked = computeBackground_Polynom(com.grad_samples, background, channel, 
-									gfit.rx, gfit.ry, degree, &error);
+		if (args->interpolation_method == INTER_POLY) {
+			interpolation_worked = computeBackground_Polynom(com.grad_samples, background, channel, gfit.rx, gfit.ry, args->degree, &error);
 		} else {
-			interpolation_worked = computeBackground_RBF(com.grad_samples, background, channel, 
-									gfit.rx, gfit.ry, smoothing, &error);
+			interpolation_worked = computeBackground_RBF(com.grad_samples, background, channel, gfit.rx, gfit.ry, args->smoothing, &error, TRUE);
 		}
 		
 		if (!interpolation_worked) {
 			free(image);
 			free(background);
-			siril_message_dialog(GTK_MESSAGE_ERROR, _("Not enough samples."),
+			queue_message_dialog(GTK_MESSAGE_ERROR, _("Not enough samples."),
 					error);
 			set_cursor_waiting(FALSE);
-			return FALSE;
+			free(args);
+			siril_add_idle(end_background, NULL);
+			return NULL;
 		}
 		/* remove background */
 		const char *c_name = vport_number_to_name(channel);
 		siril_log_message(_("Background extraction from channel %s.\n"), c_name);
-		convert_fits_to_img(&gfit, image, channel, use_dither);
-		remove_gradient(image, background, background_mean, gfit.naxes[0] * gfit.naxes[1], correction);
+		convert_fits_to_img(&gfit, image, channel, args->dither);
+		remove_gradient(image, background, background_mean, gfit.naxes[0] * gfit.naxes[1], args->correction);
 		convert_img_to_fits(image, &gfit, channel);
 	}
-	siril_log_message(_("Background with %s interpolation computed.\n"), (interpolation_method == INTER_POLY) ? "polynomial" : "RBF");
+	siril_log_message(_("Background with %s interpolation computed.\n"), (args->interpolation_method == INTER_POLY) ? "polynomial" : "RBF");
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
 	/* free memory */
 	free(image);
 	free(background);
-	return TRUE;
+	siril_add_idle(end_background, args);
+	return args;
 }
 
 /** Apply for sequence **/
@@ -913,7 +978,7 @@ static int background_image_hook(struct generic_seq_args *args, int o, int i, fi
 		if (b_args->interpolation_method == INTER_POLY){
 			interpolation_worked = computeBackground_Polynom(samples, background, channel, fit->rx, fit->ry, b_args->degree, &error);
 		} else {
-			interpolation_worked = computeBackground_RBF(samples, background, channel, fit->rx, fit->ry, b_args->smoothing, &error);
+			interpolation_worked = computeBackground_RBF(samples, background, channel, fit->rx, fit->ry, b_args->smoothing, &error, FALSE);
 		}
 		
 		if (!interpolation_worked) {
@@ -1070,13 +1135,17 @@ void on_bkg_compute_bkg_clicked(GtkButton *button, gpointer user_data) {
 	gboolean use_dither = is_dither_checked();
 	int interpolation_method = get_interpolation_method();
 
-	remove_gradient_from_image(correction, degree, smoothing, use_dither, interpolation_method);
-	background_computed = TRUE;
+	struct background_data *args = malloc(sizeof(struct background_data));
+	args->threads = com.max_thread;
+	args->from_ui = TRUE;
+	args->correction = correction;
+	args->interpolation_method = interpolation_method;
+	args->degree = (poly_order) (degree - 1);
+	args->smoothing = smoothing;
+	args->dither = use_dither;
+	args->fit = &gfit;
 
-	invalidate_stats_from_fit(&gfit);
-	adjust_cutoff_from_updated_gfit();
-	redraw(REMAP_ALL);
-	set_cursor_waiting(FALSE);
+	start_in_new_thread(remove_gradient_from_image, args);
 }
 
 void on_background_ok_button_clicked(GtkButton *button, gpointer user_data) {
@@ -1157,7 +1226,11 @@ void on_background_extraction_dialog_hide(GtkWidget *widget, gpointer user_data)
 
 	if (background_computed) {
 		siril_preview_hide();
+		background_computed = FALSE;
+	} else {
+		clear_backup();
 	}
+	gtk_widget_set_sensitive(lookup_widget("background_ok_button"), FALSE);
 }
 
 void on_background_extraction_dialog_show(GtkWidget *widget, gpointer user_data) {
