@@ -25,7 +25,7 @@ double BV_to_T(double BV);
 int pos_to_neg(fits *fit);
 void negative_processing();
 
-int equalize_cfa_fit_with_coeffs(fits *fit, float coeff1, float coeff2, int config);
+int equalize_cfa_fit_with_coeffs(fits *fit, float coeff1, float coeff2, const char *cfa_string);
 
 gpointer extract_channels(gpointer p);
 
