@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,8 +24,8 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/OS_utils.h"
-#include "algos/plateSolver.h"
 #include "algos/statistics.h"
+#include "algos/siril_wcs.h"
 #include "core/undo.h"
 #include "core/processing.h"
 #include "gui/utils.h"
@@ -34,6 +34,7 @@
 #include "gui/PSF_list.h"
 #include "gui/dialogs.h"
 #include "gui/message_dialog.h"
+#include "gui/registration_preview.h"
 #include "gui/siril_preview.h"
 #include "opencv/opencv.h"
 #include "io/single_image.h"
@@ -41,6 +42,7 @@
 #include "io/image_format_fits.h"
 
 #include "geometry.h"
+#include "astrometry_solver.h"
 
 /* this method rotates the image 180 degrees, useful after german mount flip.
  * fit->rx, fit->ry, fit->naxes[2] and fit->pdata[*] are required to be assigned correctly */
@@ -142,55 +144,49 @@ static void fits_rotate_pi(fits *fit) {
 	} else if (fit->type == DATA_FLOAT) {
 		fits_rotate_pi_float(fit);
 	}
-	invalidate_WCS_keywords(fit);
 }
 
 void mirrorx_gui(fits *fit) {
-	if (confirm_delete_wcs_keywords(fit)) {
-		set_cursor_waiting(TRUE);
-		undo_save_state(fit, _("Mirror X"));
-		mirrorx(fit, TRUE);
-		redraw(com.cvport, REMAP_ALL);
-		redraw_previews();
-		set_cursor_waiting(FALSE);
-	}
+	set_cursor_waiting(TRUE);
+	undo_save_state(fit, _("Mirror X"));
+	mirrorx(fit, TRUE);
+	redraw(REMAP_ALL);
+	redraw_previews();
+	set_cursor_waiting(FALSE);
 }
 
 void mirrory_gui(fits *fit) {
-	if (confirm_delete_wcs_keywords(fit)) {
-		set_cursor_waiting(TRUE);
-		undo_save_state(fit, _("Mirror Y"));
-		mirrory(fit, TRUE);
-		redraw(com.cvport, REMAP_ALL);
-		redraw_previews();
-		set_cursor_waiting(FALSE);
-	}
+	set_cursor_waiting(TRUE);
+	undo_save_state(fit, _("Mirror Y"));
+	mirrory(fit, TRUE);
+	redraw(REMAP_ALL);
+	redraw_previews();
+	set_cursor_waiting(FALSE);
 }
 
 static void rotate_gui(fits *fit) {
-	if (confirm_delete_wcs_keywords(fit)) {
-		static GtkToggleButton *crop_rotation = NULL;
-		double angle = gtk_spin_button_get_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_rotation")));
-		int interpolation = gtk_combo_box_get_active(
-				GTK_COMBO_BOX(lookup_widget("combo_interpolation_rotation")));
-		int cropped;
+	static GtkToggleButton *crop_rotation = NULL;
+	double angle = gtk_spin_button_get_value(
+			GTK_SPIN_BUTTON(lookup_widget("spinbutton_rotation")));
+	int interpolation = gtk_combo_box_get_active(
+			GTK_COMBO_BOX(lookup_widget("combo_interpolation_rotation")));
+	int cropped;
 
-		if (crop_rotation == NULL) {
-			crop_rotation = GTK_TOGGLE_BUTTON(
-					lookup_widget("checkbutton_rotation_crop"));
-		}
-		cropped = gtk_toggle_button_get_active(crop_rotation);
-
-		set_cursor_waiting(TRUE);
-		undo_save_state(fit, _("Rotation (%.1lfdeg, cropped=%s)"), angle,
-				cropped ? "TRUE" : "FALSE");
-		verbose_rotate_image(fit, angle, interpolation, cropped);
-		
-		redraw(com.cvport, REMAP_ALL);
-		redraw_previews();
-		set_cursor_waiting(FALSE);
+	if (crop_rotation == NULL) {
+		crop_rotation = GTK_TOGGLE_BUTTON(
+				lookup_widget("checkbutton_rotation_crop"));
 	}
+	cropped = gtk_toggle_button_get_active(crop_rotation);
+
+	set_cursor_waiting(TRUE);
+	undo_save_state(fit, _("Rotation (%.1lfdeg, cropped=%s)"), angle,
+			cropped ? "TRUE" : "FALSE");
+	verbose_rotate_image(fit, angle, interpolation, cropped);
+	
+	update_zoom_label();
+	redraw(REMAP_ALL);
+	redraw_previews();
+	set_cursor_waiting(FALSE);
 }
 
 /* These functions do not more than resize_gaussian and rotate_image
@@ -226,7 +222,7 @@ int verbose_resize_gaussian(fits *image, int toX, int toY, int interpolation) {
 	gettimeofday(&t_start, NULL);
 
 	retvalue = cvResizeGaussian(image, toX, toY, interpolation);
-	invalidate_WCS_keywords(image);
+	free_wcs(image);
 
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
@@ -267,13 +263,18 @@ int verbose_rotate_image(fits *image, double angle, int interpolation,
 	gettimeofday(&t_start, NULL);
 
 	point center = {image->rx / 2.0, image->ry / 2.0};
+
 	cvRotateImage(image, center, angle, interpolation, cropped);
 
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
 
-	invalidate_WCS_keywords(image);
-
+#ifdef HAVE_WCSLIB
+	if (image->wcslib) {
+		rotate_astrometry_data(image, center, angle, cropped);
+		load_WCS_from_memory(image);
+	}
+#endif
 	return 0;
 }
 
@@ -309,7 +310,6 @@ static void mirrorx_ushort(fits *fit, gboolean verbose) {
 		gettimeofday(&t_end, NULL);
 		show_time(t_start, t_end);
 	}
-	invalidate_WCS_keywords(fit);
 }
 
 static void mirrorx_float(fits *fit, gboolean verbose) {
@@ -344,7 +344,6 @@ static void mirrorx_float(fits *fit, gboolean verbose) {
 		gettimeofday(&t_end, NULL);
 		show_time(t_start, t_end);
 	}
-	invalidate_WCS_keywords(fit);
 }
 
 void mirrorx(fits *fit, gboolean verbose) {
@@ -353,6 +352,12 @@ void mirrorx(fits *fit, gboolean verbose) {
 	} else if (fit->type == DATA_FLOAT) {
 		mirrorx_float(fit, verbose);
 	}
+#ifdef HAVE_WCSLIB
+	if (fit->wcslib) {
+		flip_bottom_up_astrometry_data(fit);
+		load_WCS_from_memory(fit);
+	}
+#endif
 }
 
 void mirrory(fits *fit, gboolean verbose) {
@@ -370,7 +375,13 @@ void mirrory(fits *fit, gboolean verbose) {
 		gettimeofday(&t_end, NULL);
 		show_time(t_start, t_end);
 	}
-	invalidate_WCS_keywords(fit);
+
+#ifdef HAVE_WCSLIB
+	if (fit->wcslib) {
+		flip_left_right_astrometry_data(fit);
+		load_WCS_from_memory(fit);
+	}
+#endif
 }
 
 /* inplace cropping of the image in fit
@@ -412,7 +423,6 @@ static void crop_ushort(fits *fit, rectangle *bounds) {
 		show_time(t_start, t_end);
 	}
 	invalidate_stats_from_fit(fit);
-	invalidate_WCS_keywords(fit);
 }
 
 static void crop_float(fits *fit, rectangle *bounds) {
@@ -449,10 +459,12 @@ static void crop_float(fits *fit, rectangle *bounds) {
 		show_time(t_start, t_end);
 	}
 	invalidate_stats_from_fit(fit);
-	invalidate_WCS_keywords(fit);
 }
 
 int crop(fits *fit, rectangle *bounds) {
+	point shift; //need to be computed before fit rx/ry are altered by crop
+	shift.x = (double)(bounds->x);
+	shift.y = fit->ry - (double)(bounds->h) - (double)(bounds->y) - 1; // for top-bottom flip
 	if (fit->type == DATA_USHORT) {
 		crop_ushort(fit, bounds);
 	} else if (fit->type == DATA_FLOAT) {
@@ -460,6 +472,12 @@ int crop(fits *fit, rectangle *bounds) {
 	} else {
 		return -1;
 	}
+#ifdef HAVE_WCSLIB
+	if (fit->wcslib) {
+		crop_astrometry_data(fit, shift);
+		load_WCS_from_memory(fit);
+	}
+#endif
 	return 0;
 }
 
@@ -469,25 +487,23 @@ int crop(fits *fit, rectangle *bounds) {
  *  ROTATION
  */
 void siril_rotate90() {
-	if (confirm_delete_wcs_keywords(&gfit)) {
 		set_cursor_waiting(TRUE);
 		undo_save_state(&gfit, _("Rotation (90.0deg)"));
 		verbose_rotate_image(&gfit, 90.0, -1, 0);	// fast rotation, no interpolation, no crop
-		redraw(com.cvport, REMAP_ALL);
+		update_zoom_label();
+		redraw(REMAP_ALL);
 		redraw_previews();
 		set_cursor_waiting(FALSE);
-	}
 }
 
 void siril_rotate270() {
-	if (confirm_delete_wcs_keywords(&gfit)) {
 		set_cursor_waiting(TRUE);
 		undo_save_state(&gfit, _("Rotation (-90.0deg)"));
 		verbose_rotate_image(&gfit, 270.0, -1, 0);// fast rotation, no interpolation, no crop
-		redraw(com.cvport, REMAP_ALL);
+		update_zoom_label();
+		redraw(REMAP_ALL);
 		redraw_previews();
 		set_cursor_waiting(FALSE);
-	}
 }
 
 void on_button_rotation_close_clicked(GtkButton *button, gpointer user_data) {
@@ -521,7 +537,7 @@ void on_button_resample_ok_clicked(GtkButton *button, gpointer user_data) {
 				sample[1] / 100.0);
 		verbose_resize_gaussian(&gfit, toX, toY, interpolation);
 		
-		redraw(com.cvport, REMAP_ALL);
+		redraw(REMAP_ALL);
 		redraw_previews();
 		set_cursor_waiting(FALSE);
 	}
@@ -570,29 +586,23 @@ void on_button_sample_ratio_toggled(GtkToggleButton *button, gpointer user_data)
 /**************
  * CROP
  */
-void on_menu_gray_crop_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	// if astrometry exists
-	if (confirm_delete_wcs_keywords(&gfit)) {
-		undo_save_state(&gfit, _("Crop (x=%d, y=%d, w=%d, h=%d)"),
-				com.selection.x, com.selection.y, com.selection.w,
-				com.selection.h);
-		if (is_preview_active()) {
-			siril_message_dialog(GTK_MESSAGE_INFO, _("A live preview session is active"),
-					_("It is impossible to crop the image when a filter with preview session is active. "
-							"Please consider to close the filter dialog first."));
-			return;
-		}
-		crop(&gfit, &com.selection);
-		delete_selected_area();
-		reset_display_offset();
-		adjust_cutoff_from_updated_gfit();
-		redraw(com.cvport, REMAP_ALL);
-		redraw_previews();
+void siril_crop() {
+	undo_save_state(&gfit, _("Crop (x=%d, y=%d, w=%d, h=%d)"),
+			com.selection.x, com.selection.y, com.selection.w,
+			com.selection.h);
+	if (is_preview_active()) {
+		siril_message_dialog(GTK_MESSAGE_INFO, _("A live preview session is active"),
+				_("It is impossible to crop the image when a filter with preview session is active. "
+						"Please consider to close the filter dialog first."));
+		return;
 	}
-}
-
-void on_menu_gray_crop_seq_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	siril_open_dialog("crop_dialog");
+	crop(&gfit, &com.selection);
+	delete_selected_area();
+	reset_display_offset();
+	update_zoom_label();
+	adjust_cutoff_from_updated_gfit();
+	redraw(REMAP_ALL);
+	redraw_previews();
 }
 
 gint64 crop_compute_size_hook(struct generic_seq_args *args, int nb_frames) {
