@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <math.h>
 
 #include "core/siril.h"
+#include "algos/astrometry_solver.h"
 #include "io/image_format_fits.h"
 
 #ifdef HAVE_WCSLIB
@@ -48,6 +49,13 @@ gboolean has_wcs(fits *fit) {
 	return FALSE;
 }
 
+// deal with cases where wcsdata is not NULL but members are set to 0
+gboolean has_wcsdata(fits *fit) {
+	if ((fit->wcsdata.crval[0] == 0.0 && fit->wcsdata.crval[1] == 0.0)) return FALSE;
+		return TRUE;
+}
+
+
 void free_wcs(fits *fit) {
 #ifdef HAVE_WCSLIB
 	if (fit->wcslib) {
@@ -56,6 +64,7 @@ void free_wcs(fits *fit) {
 		fit->wcslib = NULL;
 	}
 #endif
+	memset(&fit->wcsdata, 0, sizeof(fit->wcsdata));
 }
 
 gboolean load_WCS_from_memory(fits *fit) {
@@ -67,14 +76,17 @@ gboolean load_WCS_from_memory(fits *fit) {
 	}
 	wcsinit(1, NAXIS, fit->wcslib, 0, 0, 0);
 
-	const char CTYPE[2][9] = {"RA---TAN", "DEC--TAN"};
+	const char CTYPE[2][9] = { "RA---TAN", "DEC--TAN" };
+	const char CUNIT[2][9] = { "deg", "deg" };
+
+	for (int i = 0; i < NAXIS; i++) {
+		strcpy(fit->wcslib->cunit[i], &CUNIT[i][0]);
+	}
 
-	double *cdij = fit->wcslib->cd;
 	double *pcij = fit->wcslib->pc;
 	for (int i = 0; i < NAXIS; i++) {
 		for (int j = 0; j < NAXIS; j++) {
-			*(cdij++) = fit->wcsdata.cd[i][j];
-			*(pcij++) = fit->wcsdata.cd[i][j];
+			*(pcij++) = fit->wcsdata.pc[i][j];
 		}
 	}
 
@@ -82,16 +94,12 @@ gboolean load_WCS_from_memory(fits *fit) {
 		fit->wcslib->crval[i] = fit->wcsdata.crval[i];
 	}
 
-	for (int i = 0; i < NAXIS; i++) {
-		fit->wcslib->crota[i] = fit->wcsdata.crota[i];
-	}
-
 	for (int i = 0; i < NAXIS; i++) {
 		fit->wcslib->crpix[i] = fit->wcsdata.crpix[i];
 	}
 
 	for (int i = 0; i < NAXIS; i++) {
-		fit->wcslib->cdelt[i] = 1;
+		fit->wcslib->cdelt[i] = fit->wcsdata.cdelt[i];
 	}
 
 	for (int i = 0; i < NAXIS; i++) {
@@ -103,7 +111,15 @@ gboolean load_WCS_from_memory(fits *fit) {
 	fit->wcslib->latpole = fit->wcsdata.crval[1];
 
 	if ((status = wcsset(fit->wcslib)) != 0) {
-		free_wcs(fit);
+	/* here we do not want to use free_wcs because
+	 * we want to keep original header, just in case */
+#ifdef HAVE_WCSLIB
+	if (fit->wcslib) {
+		if (!wcsfree(fit->wcslib))
+			free(fit->wcslib);
+		fit->wcslib = NULL;
+	}
+#endif
 		siril_debug_print("wcsset error %d: %s.\n", status, wcs_errmsg[status]);
 		return FALSE;
 	}
@@ -225,13 +241,49 @@ void wcs2pix(fits *fit, double r, double d, double *x, double *y) {
 #endif
 }
 
+/* get image center celestial coordinates */
+void center2wcs(fits *fit, double *r, double *d) {
+	*r = -1.0;
+	*d = -1.0;
+#ifdef HAVE_WCSLIB
+	int status, stat[NWCSFIX];
+	double imgcrd[NWCSFIX], phi, pixcrd[NWCSFIX], theta, world[NWCSFIX];
+
+	pixcrd[0] = (double)(fit->rx + 1) / 2.;
+	pixcrd[1] = (double)(fit->ry + 1) / 2.;
+
+	status = wcsp2s(fit->wcslib, 1, 2, pixcrd, imgcrd, &phi, &theta, world, stat);
+	if (status != 0)
+		return;
+
+	*r = world[0];
+	*d = world[1];
+#endif
+}
+
 /* get resolution in arcsec/pixel */
 double get_wcs_image_resolution(fits *fit) {
 	double resolution = -1.0;
 #ifdef HAVE_WCSLIB
 	if (fit->wcslib) {
-		double res_x = sqrt(fit->wcslib->cd[0] * fit->wcslib->cd[0] + fit->wcslib->cd[2] * fit->wcslib->cd[2]);
-		double res_y = sqrt(fit->wcslib->cd[1] * fit->wcslib->cd[1] + fit->wcslib->cd[3] * fit->wcslib->cd[3]);
+		double cd[NAXIS][NAXIS], pc[NAXIS][NAXIS];
+		double cdelt[NAXIS];
+
+		double *pcij = fit->wcslib->pc;
+		for (int i = 0; i < NAXIS; i++) {
+			for (int j = 0; j < NAXIS; j++) {
+				pc[i][j] = *(pcij++);
+			}
+		}
+
+		for (int i = 0; i < NAXIS; i++) {
+			cdelt[i] = fit->wcslib->cdelt[i];
+		}
+
+		wcs_pc_to_cd(pc, cdelt, cd);
+
+		double res_x = sqrt(cd[0][0] * cd[0][0] + cd[1][0] * cd[1][0]);
+		double res_y = sqrt(cd[0][1] * cd[0][1] + cd[1][1] * cd[1][1]);
 		resolution = (res_x + res_y) * 0.5;
 	}
 #endif
@@ -241,18 +293,3 @@ double get_wcs_image_resolution(fits *fit) {
 	}
 	return resolution;
 }
-
-double *get_wcs_crval(fits *fit) {
-#ifdef HAVE_WCSLIB
-	static double ret[NWCSFIX] = { 0 };
-	if (fit->wcslib) {
-		for (int i = 0; i < NAXIS; i++) {
-			ret[i] = fit->wcslib->crval[i];
-		}
-	}
-	return ret;
-#else
-	return NULL;
-#endif
-}
-
