@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -55,54 +55,54 @@ broad_filter broad_band_filters[] = {
  * I couldn't find why gamma is 0.8 anywhere. */
 static double Gamma = 0.80;
 
-void wavelength_to_RGB(double wavelength, GdkRGBA *rgb){
+void wavelength_to_RGB(double wavelength, GdkRGBA *rgb) {
 	double factor;
 	double Red,Green,Blue;
 
-	if((wavelength >= 380) && (wavelength<440)){
+	if (wavelength >= 380 && wavelength < 440) {
 		Red = -(wavelength - 440) / (440 - 380);
 		Green = 0.0;
 		Blue = 1.0;
-	}else if((wavelength >= 440) && (wavelength<490)){
+	} else if (wavelength >= 440 && wavelength < 490) {
 		Red = 0.0;
 		Green = (wavelength - 440) / (490 - 440);
 		Blue = 1.0;
-	}else if((wavelength >= 490) && (wavelength<510)){
+	} else if (wavelength >= 490 && wavelength < 510) {
 		Red = 0.0;
 		Green = 1.0;
 		Blue = -(wavelength - 510) / (510 - 490);
-	}else if((wavelength >= 510) && (wavelength<580)){
+	} else if (wavelength >= 510 && wavelength < 580) {
 		Red = (wavelength - 510) / (580 - 510);
 		Green = 1.0;
 		Blue = 0.0;
-	}else if((wavelength >= 580) && (wavelength<645)){
+	} else if (wavelength >= 580 && wavelength < 645) {
 		Red = 1.0;
 		Green = -(wavelength - 645) / (645 - 580);
 		Blue = 0.0;
-	}else if((wavelength >= 645) && (wavelength<781)){
+	} else if (wavelength >= 645 && wavelength < 781) {
 		Red = 1.0;
 		Green = 0.0;
 		Blue = 0.0;
-	}else{
+	} else {
 		Red = 0.0;
 		Green = 0.0;
 		Blue = 0.0;
 	};
 
 	// Let the intensity fall off near the vision limits
-	if((wavelength >= 380) && (wavelength<420)){
+	if (wavelength >= 380 && wavelength < 420) {
 		factor = 0.3 + 0.7*(wavelength - 380) / (420 - 380);
-	}else if((wavelength >= 420) && (wavelength<701)){
+	} else if (wavelength >= 420 && wavelength < 701) {
 		factor = 1.0;
-	}else if((wavelength >= 701) && (wavelength<781)){
+	} else if (wavelength >= 701 && wavelength < 781) {
 		factor = 0.3 + 0.7*(780 - wavelength) / (780 - 700);
-	}else{
+	} else {
 		factor = 0.0;
 	};
 
 	// Don't want 0^x = 1 for x <> 0
-	rgb->red = Red==0.0 ? 0 : pow(Red * factor, Gamma);
-	rgb->green = Green==0.0 ? 0 : pow(Green * factor, Gamma);
-	rgb->blue = Blue==0.0 ? 0 : pow(Blue * factor, Gamma);
+	rgb->red = Red == 0.0 ? 0 : pow(Red * factor, Gamma);
+	rgb->green = Green == 0.0 ? 0 : pow(Green * factor, Gamma);
+	rgb->blue = Blue == 0.0 ? 0 : pow(Blue * factor, Gamma);
 	rgb->alpha = 1.0;
 }
