@@ -76,6 +76,7 @@ int round_to_ceiling_multiple(int x, int factor);
 BYTE conv_to_BYTE(double x);
 int truncate_to_int32(uint64_t x);
 WORD truncate_to_WORD(int x);
+BYTE truncate_to_BYTE(WORD x);
 int set_int_in_interval(int val, int low, int high);
 float set_float_in_interval(float val, float low, float high);
 double set_double_in_interval(double val, double low, double high);
@@ -130,6 +131,11 @@ gchar *siril_get_file_info(const gchar *filename, GdkPixbuf *pixbuf);
 gchar *siril_truncate_str(gchar *str, gint size);
 char **glist_to_array(GList *list, int *arg_count);
 gchar* url_cleanup(const gchar *uri_string);
+void remove_spaces_from_str(gchar *s);
+#if !GLIB_CHECK_VERSION(2,68,0)
+guint g_string_replace(GString *string, const gchar *find, const gchar *replace,
+		guint limit);
+#endif
 
 /****************** quantize.h ***************/
 int siril_fits_img_stats_ushort(WORD *array, long nx, long ny, int nullcheck,
@@ -164,16 +170,6 @@ int writeseqfile(sequence *seq);
 gboolean existseq(const char *name);
 int buildseqfile(sequence *seq, int force_recompute);
 
-/****************** registration_preview.h ******************/
-void redraw_previews();
-void set_preview_area(int preview_area, int centerX, int centerY);
-void init_mouse();
-void adjust_reginfo();
-void on_spinbut_shift_value_change(GtkSpinButton *spinbutton,
-		gpointer user_data);
-void test_and_allocate_reference_image(int vport);
-void enable_view_reference_checkbox(gboolean status);
-
 /****************** statistics_list.h ******************/
 void computeStat();
 
