@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -147,6 +147,15 @@ gchar* siril_world_cs_delta_format(SirilWorldCS *world_cs, const gchar *format)
 	int degree = (int) dec;
 	int min = abs((int) ((dec - degree) * 60.0));
 	double sec = (fabs((dec - degree) * 60.0) - min) * 60.0;
+	if (round(sec) >= 60.) {
+		sec -= 60.;
+		min += 1;
+	}
+	if (min >= 60) {
+		min -= 60;
+		degree += 1;
+	}
+	if (degree >= 360) degree = 0;
 
 	gchar *ptr = g_strrstr(format, "lf");
 	if (ptr) { // floating point for second
@@ -167,6 +176,15 @@ gchar* siril_world_cs_alpha_format(SirilWorldCS *world_cs, const gchar *format)
 	int hour = (int)(ra / 15.0);
 	int min = (int)(((ra / 15.0) - hour) * 60.0);
 	double sec = ((((ra / 15.0) - hour) * 60.0) - min) * 60.0;
+	if (round(sec) >= 60.) {
+		sec -= 60.;
+		min += 1;
+	}
+	if (min >= 60) {
+		min -= 60;
+		hour += 1;
+	}
+	if (hour >= 24) hour = 0;
 
 	gchar *ptr = g_strrstr(format, "lf");
 	if (ptr) { // floating point for second
