@@ -264,13 +264,18 @@ void on_rgradient_Apply_clicked(GtkButton *button, gpointer user_data) {
 void on_button_rgradient_selection_clicked(GtkButton *button, gpointer user_data) {
 	if (com.selection.h && com.selection.w) {
 		psf_star *result = psf_get_minimisation(&gfit, 0, &com.selection, FALSE, FALSE, TRUE, TRUE);
-		gchar *x0 = g_strdup_printf("%.3lf", result->x0 + com.selection.x);
-		gtk_entry_set_text(GTK_ENTRY(lookup_widget("entry_rgradient_xc")), x0);
-		gchar *y0 = g_strdup_printf("%.3lf", com.selection.y + com.selection.h - result->y0);
-		gtk_entry_set_text(GTK_ENTRY(lookup_widget("entry_rgradient_yc")), y0);
-
-		g_free(x0);
-		g_free(y0);
-		free_psf(result);
+		if (result != NULL) {
+			gchar *x0 = g_strdup_printf("%.3lf", result->x0 + com.selection.x);
+			gtk_entry_set_text(GTK_ENTRY(lookup_widget("entry_rgradient_xc")), x0);
+			gchar *y0 = g_strdup_printf("%.3lf", com.selection.y + com.selection.h - result->y0);
+			gtk_entry_set_text(GTK_ENTRY(lookup_widget("entry_rgradient_yc")), y0);
+			g_free(x0);
+			g_free(y0);
+			free_psf(result);
+		} else {
+			siril_message_dialog(GTK_MESSAGE_ERROR, _("Center coordinate selection error"),
+				_("No valid PSF found within selection."));
+		}
 	}
 }
+
