@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include "gui/histogram.h"
 #include "gui/dialogs.h"
 #include "gui/siril_preview.h"
+#include "gui/registration_preview.h"
 
 #include "saturation.h"
 
@@ -311,7 +312,7 @@ void on_satu_undo_clicked(GtkButton *button, gpointer user_data) {
 
 	copy_backup_to_gfit();
 	adjust_cutoff_from_updated_gfit();
-	redraw(com.cvport, REMAP_ALL);
+	redraw(REMAP_ALL);
 	redraw_previews();
 	set_cursor_waiting(FALSE);
 }
