@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,18 +25,18 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/OS_utils.h"
-#include "io/single_image.h"
-#include "io/sequence.h"
+#include "core/siril_world_cs.h"
 #include "gui/utils.h"
 #include "gui/callbacks.h"
 #include "gui/image_display.h"
-#include "gui/image_interactions.h"
 #include "gui/dialogs.h"
 #include "gui/message_dialog.h"
 #include "gui/PSF_list.h"
 #include "gui/progress_and_log.h"
+#include "algos/siril_wcs.h"
 #include "algos/PSF.h"
 #include "algos/star_finder.h"
+#include "algos/ccd-inspector.h"
 
 static GtkListStore *liststore_stars = NULL;
 
@@ -162,45 +162,45 @@ static void gdouble_rmse_cell_data_function(GtkTreeViewColumn *col,
 
 static void get_stars_list_store() {
 	if (liststore_stars == NULL)
-		liststore_stars = GTK_LIST_STORE(gtk_builder_get_object(builder, "liststore_stars"));
+		liststore_stars = GTK_LIST_STORE(gtk_builder_get_object(gui.builder, "liststore_stars"));
 
 	GtkTreeViewColumn *col;
 	GtkCellRenderer *cell;
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn7"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_x0"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn7"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_x0"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_x0_cell_data_function, NULL, NULL);
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn8"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_y0"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn8"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_y0"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_y0_cell_data_function, NULL, NULL);
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn9"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_fwhmx"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn9"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_fwhmx"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_fwhmx_cell_data_function, NULL, NULL);
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn10"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_fwhmy"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn10"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_fwhmy"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_fwhmy_cell_data_function, NULL, NULL);
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn_mag"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_mag"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn_mag"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_mag"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_mag_cell_data_function, NULL, NULL);
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn14"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_r"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn14"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_r"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_r_cell_data_function, NULL, NULL);
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn6"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_angle"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn6"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_angle"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_angle_cell_data_function, NULL, NULL);
 
-	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(builder, "treeviewcolumn15"));
-	cell = GTK_CELL_RENDERER(gtk_builder_get_object(builder, "cell_rmse"));
+	col = GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn15"));
+	cell = GTK_CELL_RENDERER(gtk_builder_get_object(gui.builder, "cell_rmse"));
 	gtk_tree_view_column_set_cell_data_func(col, cell, gdouble_rmse_cell_data_function, NULL, NULL);
 }
 
-static void display_PSF(fitted_PSF **result) {
+static void display_PSF(psf_star **result) {
 	if (result) {
 		gchar *msg;
 		int i = 0;
@@ -284,8 +284,8 @@ static void display_status() {
 }
 
 static void remove_selected_star(int index) {
-	GtkTreeSelection *selection = GTK_TREE_SELECTION(gtk_builder_get_object(builder, "treeview-selection"));
-	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(gtk_builder_get_object(builder, "Stars_stored")));
+	GtkTreeSelection *selection = GTK_TREE_SELECTION(gtk_builder_get_object(gui.builder, "treeview-selection"));
+	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(gtk_builder_get_object(gui.builder, "Stars_stored")));
 	GtkTreeIter iter;
 
 	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
@@ -303,7 +303,7 @@ static void remove_all_stars(){
 	clear_stars_list();
 	com.selected_star = -1;
 	display_status();
-	redraw(com.cvport, REMAP_NONE);
+	redraw(REDRAW_OVERLAY);
 }
 
 static int save_list(gchar *filename) {
@@ -346,13 +346,13 @@ static int save_list(gchar *filename) {
 					"%d\t%d\t%10.6f\t%10.6f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%3.2f\t%10.3e\t%10.2f%s",
 					i + 1, com.stars[i]->layer, com.stars[i]->B, com.stars[i]->A,
 					com.stars[i]->xpos, com.stars[i]->ypos, com.stars[i]->fwhmx_arcsec,
-					com.stars[i]->fwhmy_arcsec, com.stars[i]->angle, com.stars[i]->rmse, com.stars[i]->mag, SIRIL_EOL);
+					com.stars[i]->fwhmy_arcsec, com.stars[i]->angle, com.stars[i]->rmse, com.stars[i]->mag + com.magOffset, SIRIL_EOL);
 		} else {
 			buffer = g_strdup_printf(
 					"%d\t%d\t%10.6f\t%10.6f\t%10.2f\t%10.2f\t%10.2f\t%10.2f\t%3.2f\t%10.3e\t%10.2f%s",
 					i + 1, com.stars[i]->layer, com.stars[i]->B, com.stars[i]->A,
 					com.stars[i]->xpos, com.stars[i]->ypos, com.stars[i]->fwhmx,
-					com.stars[i]->fwhmy, com.stars[i]->angle, com.stars[i]->rmse, com.stars[i]->mag, SIRIL_EOL);
+					com.stars[i]->fwhmy, com.stars[i]->angle, com.stars[i]->rmse, com.stars[i]->mag + com.magOffset, SIRIL_EOL);
 		}
 
 		if (!g_output_stream_write_all(output_stream, buffer, strlen(buffer), NULL, NULL, &error)) {
@@ -406,13 +406,13 @@ static void save_stars_dialog() {
 
 /********************* public ***********************/
 
-void add_star_to_list(fitted_PSF *star) {
+void add_star_to_list(psf_star *star) {
 	static GtkTreeSelection *selection = NULL;
 	GtkTreeIter iter;
 
 	get_stars_list_store();
 	if (!selection)
-		selection = GTK_TREE_SELECTION(gtk_builder_get_object(builder, "treeview-selection"));
+		selection = GTK_TREE_SELECTION(gtk_builder_get_object(gui.builder, "treeview-selection"));
 	if (star == NULL) {
 		gtk_list_store_clear(liststore_stars);
 		return;		// just clear the list
@@ -430,7 +430,7 @@ void add_star_to_list(fitted_PSF *star) {
 			COLUMN_Y0, star->ypos,
 			COLUMN_FWHMX, fwhmx,
 			COLUMN_FWHMY, fwhmy,
-			COLUMN_MAG, star->mag,
+			COLUMN_MAG, star->mag + com.magOffset,
 			COLUMN_ROUNDNESS, fwhmy / fwhmx,
 			COLUMN_ANGLE, star->angle,
 			COLUMN_RMSE, star->rmse,
@@ -440,7 +440,7 @@ void add_star_to_list(fitted_PSF *star) {
 	display_status();
 }
 
-void fill_stars_list(fits *fit, fitted_PSF **stars) {
+void fill_stars_list(fits *fit, psf_star **stars) {
 	int i = 0;
 	if (stars == NULL)
 		return;
@@ -455,11 +455,11 @@ void fill_stars_list(fits *fit, fitted_PSF **stars) {
 	com.selected_star = -1;
 }
 
-void refresh_star_list(fitted_PSF **star){
+void refresh_star_list(psf_star **star){
 	get_stars_list_store();
 	gtk_list_store_clear(liststore_stars);
 	fill_stars_list(&gfit, com.stars);
-	redraw(com.cvport, REMAP_NONE);
+	redraw(REDRAW_OVERLAY);
 }
 
 void clear_stars_list() {
@@ -468,6 +468,8 @@ void clear_stars_list() {
 			get_stars_list_store();
 			gtk_list_store_clear(liststore_stars);
 		}
+		g_mutex_lock(&com.mutex);
+
 		if (com.stars[0]) {
 			/* freeing found stars. It must not be done when the only star in
 			 * com.stars is the same as com.seq.imgparam[xxx].fwhm, as set in
@@ -475,19 +477,18 @@ void clear_stars_list() {
 			if (com.stars[1] || !com.star_is_seqdata) {
 				int i = 0;
 				while (i < MAX_STARS && com.stars[i])
-					free(com.stars[i++]);
+					free_psf(com.stars[i++]);
 			}
 		}
 		free(com.stars);
 		com.stars = NULL;
+		g_mutex_unlock(&com.mutex);
 	}
 	com.star_is_seqdata = FALSE;
 }
 
 void pick_a_star() {
-	int layer = match_drawing_area_widget(com.vport[com.cvport], FALSE);
-	int new_index;
-
+	int layer = match_drawing_area_widget(gui.view[gui.cvport].drawarea, FALSE);
 	if (layer != -1) {
 		if (!(com.selection.h && com.selection.w))
 			return;
@@ -496,14 +497,109 @@ void pick_a_star() {
 					_("To determine the PSF, please make a selection around a star."));
 			return;
 		}
-		fitted_PSF *new_star = add_star(&gfit, layer, &new_index);
+		int new_index;
+		psf_star *new_star = add_star(&gfit, layer, &new_index);
 		if (new_star) {
 			add_star_to_list(new_star);
 			siril_open_dialog("stars_list_window");
 		} else
 			return;
 	}
-	redraw(com.cvport, REMAP_NONE);
+	redraw(REDRAW_OVERLAY);
+}
+
+static gchar *build_wcs_url(gchar *ra, gchar *dec) {
+	if (!has_wcs(&gfit)) return NULL;
+
+	double resolution = get_wcs_image_resolution(&gfit);
+
+	gchar *tol = g_strdup_printf("%lf", resolution * 3600 * 15);
+
+	GString *url = g_string_new("https://simbad.u-strasbg.fr/simbad/sim-coo?Coord=");
+	url = g_string_append(url, ra);
+	url = g_string_append(url, dec);
+	url = g_string_append(url, "&Radius=");
+	url = g_string_append(url, tol);
+	url = g_string_append(url, "&Radius.unit=arcsec");
+	url = g_string_append(url, "#lab_basic");
+
+	gchar *simbad_url = g_string_free(url, FALSE);
+	gchar *cleaned_url = url_cleanup(simbad_url);
+
+	g_free(tol);
+	g_free(simbad_url);
+
+	return cleaned_url;
+}
+
+static const char *SNR_quality(double SNR) {
+	if (SNR > 40.0) return _("Excellent");
+	if (SNR > 25.0) return _("Good");
+	if (SNR > 15.0) return _("Fair");
+	if (SNR > 10.0) return _("Poor");
+	if (SNR > 0.0) return _("Bad");
+	else return _("N/A");
+}
+
+void popup_psf_result(psf_star *result, rectangle *area) {
+	gchar *msg, *coordinates, *url = NULL;
+	const char *str;
+	if (com.magOffset > 0.0)
+		str = _("true reduced");
+	else
+		str = _("relative");
+
+	double x = result->x0 + area->x;
+	double y = area->y + area->h - result->y0;
+	if (has_wcs(&gfit)) {
+		double world_x, world_y;
+		SirilWorldCS *world_cs;
+
+		pix2wcs(&gfit, x, (double) gfit.ry - y, &world_x, &world_y);
+		world_cs = siril_world_cs_new_from_a_d(world_x, world_y);
+		if (world_cs) {
+			gchar *ra = siril_world_cs_alpha_format(world_cs, "%02d %02d %.3lf");
+			gchar *dec = siril_world_cs_delta_format(world_cs, "%c%02d %02d %.3lf");
+
+			url = build_wcs_url(ra, dec);
+
+			g_free(ra);
+			g_free(dec);
+
+			ra = siril_world_cs_alpha_format(world_cs, " %02dh%02dm%02ds");
+			dec = siril_world_cs_delta_format(world_cs, "%c%02d°%02d\'%02d\"");
+
+			coordinates = g_strdup_printf("x0=%.2fpx\t%s J2000\n\t\ty0=%.2fpx\t%s J2000", x, ra, y, dec);
+
+			g_free(ra);
+			g_free(dec);
+			siril_world_cs_unref(world_cs);
+		} else {
+			coordinates = g_strdup_printf("x0=%.2fpx\n\t\ty0=%.2fpx", x, y);
+		}
+	} else {
+		coordinates = g_strdup_printf("x0=%.2fpx\n\t\ty0=%.2fpx", x, y);
+	}
+
+	double fwhmx, fwhmy;
+	char *unts;
+	get_fwhm_as_arcsec_if_possible(result, &fwhmx, &fwhmy, &unts);
+	msg = g_strdup_printf(_("Centroid Coordinates:\n\t\t%s\n\n"
+				"Full Width Half Maximum:\n\t\tFWHMx=%.2f%s\n\t\tFWHMy=%.2f%s\n\n"
+				"Angle:\n\t\t%0.2fdeg\n\n"
+				"Background Value:\n\t\tB=%.6f\n\n"
+				"Maximal Intensity:\n\t\tA=%.6f\n\n"
+				"Magnitude (%s):\n\t\tm=%.4f\u00B1%.4f\n\n"
+				"Signal-to-noise ratio:\n\t\tSNR=%.1fdB (%s)\n\n"
+				"RMSE:\n\t\tRMSE=%.3e"),
+			coordinates, fwhmx, unts, fwhmy, unts,
+			result->angle, result->B, result->A, str,
+			result->mag + com.magOffset, result->s_mag, result->SNR,
+			SNR_quality(result->SNR), result->rmse);
+	show_data_dialog(msg, _("PSF and quick photometry results"), NULL, url);
+	g_free(coordinates);
+	g_free(msg);
+	g_free(url);
 }
 
 /***************** callbacks ****************/
@@ -531,7 +627,7 @@ void on_treeview_cursor_changed(GtkTreeView *tree_view,
 
 		com.selected_star = get_index_of_selected_star(x0, y0);
 		display_status();
-		redraw(com.cvport, REMAP_NONE);
+		redraw(REDRAW_OVERLAY);
 	}
 }
 
@@ -558,24 +654,7 @@ void on_remove_button_clicked(GtkButton *button, gpointer user_data) {
 
 void on_remove_all_button_clicked(GtkButton *button, gpointer user_data) {
 	remove_all_stars();
-}
-
-void on_process_starfinder_button_clicked(GtkButton *button, gpointer user_data) {
-	int nbstars = 0;
-	int layer = com.cvport == RGB_VPORT ? GLAYER : com.cvport;
-	if (!single_image_is_loaded() && !sequence_is_loaded()) {
-		siril_log_color_message(_("Load an image first, aborted.\n"), "red");
-		return;
-	}
-	set_cursor_waiting(TRUE);
-
-	confirm_peaker_GUI(); //making sure the spin buttons values are read even without confirmation
-	delete_selected_area();
-	com.stars = peaker(&gfit, layer, &com.starfinder_conf, &nbstars, NULL, TRUE, FALSE);
-	siril_log_message(_("Found %d stars in image, channel #%d\n"), nbstars, layer);
-	if (com.stars)
-		refresh_star_list(com.stars);
-	set_cursor_waiting(FALSE);
+	clear_sensor_tilt();
 }
 
 void on_export_button_clicked(GtkButton *button, gpointer user_data) {
