@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,6 +24,7 @@
 #include "core/exif.h"
 #include "gui/utils.h"
 #include "gui/histogram.h"
+#include "gui/progress_and_log.h"
 #include "io/ser.h"
 #include "io/image_format_fits.h"
 
@@ -65,7 +66,6 @@ static gboolean end_update_preview_cb(gpointer p) {
 
 	stop_processing_thread();
 
-	int bytes;
 	const char *bytes_str;
 	char *size_str = NULL;
 	char *name_str = NULL;
@@ -75,19 +75,21 @@ static gboolean end_update_preview_cb(gpointer p) {
 	fileChooserPreview *preview = args->preview;
 
 	if (!preview_allocated || !preview || !(GTK_IS_IMAGE((preview->image)))) {
+		set_cursor_waiting(FALSE);
 		return FALSE;
 	}
 
 	name_str = g_path_get_basename(args->filename);
 
 	if (!args->file_info) {
+		set_cursor_waiting(FALSE);
 		return FALSE;
 	}
 	type = g_file_info_get_file_type(args->file_info);
 
 	/* try to read file size */
 	if (args->pixbuf && (bytes_str = gdk_pixbuf_get_option(args->pixbuf, "tEXt::Thumb::Size")) != NULL) {
-		bytes = g_ascii_strtoll(bytes_str, NULL, 10);
+		int bytes = g_ascii_strtoll(bytes_str, NULL, 10);
 		size_str = g_format_size(bytes);
 	} else {
 		if (type == G_FILE_TYPE_REGULAR) {
@@ -135,6 +137,7 @@ static gboolean end_update_preview_cb(gpointer p) {
 
 	free(args);
 	args = NULL;
+	set_cursor_waiting(FALSE);
 	return FALSE;
 }
 
