@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -23,6 +23,8 @@
 #include "core/initfile.h"
 #include "core/siril_language.h"
 #include "algos/photometry.h"
+#include "algos/astrometry_solver.h"
+#include "algos/annotate.h"
 #include "gui/callbacks.h"
 #include "gui/utils.h"
 #include "gui/message_dialog.h"
@@ -51,6 +53,8 @@ static preferences pref_init = {
 				.w = 0,
 				.h = 0
 		},
+		.pan_position = -1,
+		.is_extended = TRUE,
 		.is_maximized = FALSE,
 		.prepro_cfa = FALSE,
 		.prepro_equalize_cfa = TRUE,
@@ -118,6 +122,8 @@ static preferences pref_init = {
 				.gain = 2.3,
 				.inner = 20.0,
 				.outer = 30.0,
+				.aperture = 10.0,
+				.force_radius = FALSE,
 				.minval = 0,
 				.maxval = 60000,
 		},
@@ -128,6 +134,9 @@ static preferences pref_init = {
 		.catalog[3] = TRUE,
 		.catalog[4] = TRUE,
 		.catalog[5] = TRUE,
+		.catalog[6] = TRUE,
+		.position_compass = 1,
+		.wcs_formalism = WCS_FORMALISM_1,
 		{ // stack_config
 				.method = 0,
 				.normalisation_method = ADDITIVE_SCALING,
@@ -212,6 +221,9 @@ static void update_astrometry_preferences() {
 	com.pref.catalog[3] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_ldn")));
 	com.pref.catalog[4] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_sh2")));
 	com.pref.catalog[5] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_stars")));
+	com.pref.catalog[6] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_user-catalogue")));
+	com.pref.position_compass = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("compass_combobox")));
+	com.pref.wcs_formalism = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("wcs_formalism_combobox")));
 }
 
 static void update_prepro_preferences() {
@@ -257,6 +269,8 @@ static void update_photometry_preferences() {
 	com.pref.phot_set.gain = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinGain")));
 	com.pref.phot_set.inner = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinInner")));
 	com.pref.phot_set.outer = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinOuter")));
+	com.pref.phot_set.aperture = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinAperture")));
+	com.pref.phot_set.force_radius = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("photometry_force_radius_button")));
 	com.pref.phot_set.minval = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinMinPhot")));
 	com.pref.phot_set.maxval = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinMaxPhot")));
 }
@@ -336,6 +350,11 @@ static void update_misc_preferences() {
 	com.pref.check_update = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("miscAskUpdateStartup")));
 }
 
+void on_photometry_force_radius_button_toggled(GtkToggleButton *button, gpointer user_data) {
+	GtkWidget *spin = (GtkWidget *)user_data;
+	gtk_widget_set_sensitive(spin, !gtk_toggle_button_get_active(button));
+}
+
 void on_checkbutton_cam_toggled(GtkToggleButton *cam_button, gpointer user_data) {
 	GtkToggleButton *auto_button = GTK_TOGGLE_BUTTON(lookup_widget("checkbutton_auto"));
 
@@ -389,12 +408,16 @@ void set_GUI_photometry() {
 	if (com.pref.phot_set.outer > 0.0) {
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinOuter")), com.pref.phot_set.outer);
 	}
+	if (com.pref.phot_set.aperture > 0.0) {
+		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinAperture")), com.pref.phot_set.aperture);
+	}
 	if (com.pref.phot_set.minval >= 0.0) {
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinMinPhot")), (gdouble) com.pref.phot_set.minval);
 	}
 	if (com.pref.phot_set.maxval >= 0.0) {
 		gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinMaxPhot")), (gdouble) com.pref.phot_set.maxval);
 	}
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("photometry_force_radius_button")), !com.pref.phot_set.force_radius);
 }
 
 void initialize_path_directory(const gchar *path) {
@@ -457,9 +480,6 @@ void set_GUI_compression() {
 		if (com.pref.comp.fits_method == HCOMPRESS_COMP) {
 			gtk_spin_button_set_value(hscale, com.pref.comp.fits_hcompress_scale);
 		}
-		siril_log_message(_("FITS compression enabled\n"), com.pref.ext);
-	} else {
-		siril_log_message(_("FITS compression disabled\n"));
 	}
 }
 
@@ -580,6 +600,8 @@ void on_check_button_pref_bias_toggled(GtkToggleButton *togglebutton, gpointer u
 	}
 }
 
+static gboolean from_prefs_init = FALSE;
+
 void on_spinInner_value_changed(GtkSpinButton *inner, gpointer user_data) {
 	GtkSpinButton *outer;
 	double in, out;
@@ -588,7 +610,7 @@ void on_spinInner_value_changed(GtkSpinButton *inner, gpointer user_data) {
 	in = gtk_spin_button_get_value(inner);
 	out = gtk_spin_button_get_value(outer);
 
-	if (in >= out) {
+	if (!from_prefs_init && in >= out) {
 		siril_message_dialog(GTK_MESSAGE_ERROR, _("Wrong value"),
 				_("Inner radius value must be less than outer. Please change the value."));
 	}
@@ -602,7 +624,7 @@ void on_spinOuter_value_changed(GtkSpinButton *outer, gpointer user_data) {
 	in = gtk_spin_button_get_value(inner);
 	out = gtk_spin_button_get_value(outer);
 
-	if (in >= out) {
+	if (!from_prefs_init && in >= out) {
 		siril_message_dialog(GTK_MESSAGE_ERROR, _("Wrong value"),
 				_("Inner radius value must be less than outer. Please change the value."));
 	}
@@ -645,6 +667,9 @@ static void set_preferences_ui(preferences *pref) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_ldn")), pref->catalog[3]);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_sh2")), pref->catalog[4]);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_stars")), pref->catalog[5]);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("check_button_user-catalogue")), pref->catalog[6]);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(lookup_widget("compass_combobox")), pref->position_compass);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(lookup_widget("wcs_formalism_combobox")), pref->wcs_formalism);
 
 	/* tab 5*/
 	if (pref->prepro_bias_lib && (g_file_test(pref->prepro_bias_lib, G_FILE_TEST_EXISTS))) {
@@ -699,8 +724,12 @@ static void set_preferences_ui(preferences *pref) {
 	gtk_entry_set_text(GTK_ENTRY(lookup_widget("xtrans_sample_h")), tmp);
 
 	/* tab 6 */
+	from_prefs_init = TRUE;
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinOuter")), pref->phot_set.outer);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinInner")), pref->phot_set.inner);
+	from_prefs_init = FALSE;
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinAperture")), pref->phot_set.aperture);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lookup_widget("photometry_force_radius_button")), !pref->phot_set.force_radius);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinGain")), pref->phot_set.gain);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinMinPhot")), pref->phot_set.minval);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(lookup_widget("spinMaxPhot")), pref->phot_set.maxval);
@@ -780,19 +809,32 @@ void initialize_default_preferences() {
 	com.pref.swap_dir = g_strdup(g_get_tmp_dir());
 }
 
+static int type_of_closure = 0;
+
+void on_settings_window_hide(GtkWidget *widget, gpointer user_data) {
+	if (type_of_closure == 1) {
+		free_preferences(&com.pref);
+		dump_ui_to_global_var();
+
+		initialize_FITS_name_entries(); // To update UI with new preferences
+		refresh_star_list(com.stars); // To update star list with new preferences
+		if (com.found_object)
+			force_to_refresh_catalogue_list();
+		save_main_window_state();
+		writeinitfile();
+	} else {
+		set_preferences_ui(&com.pref);
+	}
+	type_of_closure = 0;
+}
+
 void on_apply_settings_button_clicked(GtkButton *button, gpointer user_data) {
-	free_preferences(&com.pref);
-	dump_ui_to_global_var();
+	type_of_closure = 1;
 
-	initialize_FITS_name_entries(); // To update UI with new preferences
-	refresh_star_list(com.stars); // To update star list with new preferences
-	save_main_window_state();
 	siril_close_dialog("settings_window");
-	writeinitfile();
 }
 
 void on_cancel_settings_button_clicked(GtkButton *button, gpointer user_data) {
-	set_preferences_ui(&com.pref);
 	siril_close_dialog("settings_window");
 }
 
